<?php
namespace KnitPay\Gateways\Easebuzz;

use Pronamic\WordPress\Pay\Core\Gateway as Core_Gateway;
use Exception;
use Pronamic\WordPress\Pay\Payments\Payment;
use WP_Error;


/**
 * Title: Easebuzz Gateway
 * Copyright: 2020 Knit Pay
 *
 * @author Knit Pay
 * @version 1.0.0
 * @since 1.2.0
 */
class Gateway extends Core_Gateway {


	/**
	 * Client.
	 *
	 * @var Client
	 */

	/**
	 * Constructs and initializes an Easebuzz gateway
	 *
	 * @param Config $config
	 *            Config.
	 */
	public function __construct( Config $config ) {
		parent::__construct( $config );

		$this->set_method( self::METHOD_HTTP_REDIRECT );

		// Supported features.
		$this->supports = array(
			'payment_status_request',
		);

		$this->ENV = 'prod';
		if ( self::MODE_TEST === $config->mode ) {
			$this->ENV = 'test';
		}
	}

	/**
	 * Get supported payment methods
	 *
	 * @see Pronamic_WP_Pay_Gateway::get_supported_payment_methods()
	 */
	public function get_supported_payment_methods() {
		return array(
			PaymentMethods::BANK_TRANSFER,
			PaymentMethods::CREDIT_CARD,
			PaymentMethods::MAESTRO,
			PaymentMethods::EASEBUZZ,
		);
	}

	/**
	 * Start.
	 *
	 * @see Core_Gateway::start()
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function start( Payment $payment ) {
		$payment_currency = $payment->get_total_amount()->get_currency()->get_alphabetic_code();
		if ( isset( $payment_currency ) && 'INR' !== $payment_currency ) {
			$currency_error = 'Easebuzz only accepts payments in Indian Rupees. If you are a store owner, kindly activate INR currency for ' . $payment->get_source() . ' plugin.';
			$this->error    = new WP_Error( 'easebuzz_error', $currency_error );
			return;
		}

		include_once 'easebuzz-lib/easebuzz_payment_gateway.php';

		$merchant_key  = $this->config->merchant_key;
		$merchant_salt = $this->config->merchant_salt;

		$txnid = $payment->get_id();

		$amount       = $payment->get_total_amount()->format();
		$product_info = $payment->get_description();

		$customer = $payment->get_customer();
		if ( null !== $customer->get_name() ) {
			$first_name = $customer->get_name()->get_first_name();
			$last_name  = $customer->get_name()->get_last_name();
		}
		if ( empty( $first_name ) ) {
			$first_name = 'Empty';
		}
		$email = $customer->get_email();
		$phone = '9999999999';
		if ( null !== $customer->get_phone() ) {
			$phone = $customer->get_phone();
		}

		$billing_address = $payment->get_billing_address();
		if ( null !== $billing_address ) {
			$address1 = $billing_address->get_line_1();
			$address2 = $billing_address->get_line_2();
			$city     = $billing_address->get_city();
			$state    = $billing_address->get_region();
			$country  = $billing_address->get_country();
			$zipcode  = $billing_address->get_postal_code();
		}

		$surl = $payment->get_return_url();
		$furl = $payment->get_return_url();

		$easebuzzObj = new Easebuzz( $merchant_key, $merchant_salt, $this->ENV );

		$postData = array(
			'txnid'       => $txnid,
			'amount'      => $amount,
			'firstname'   => $first_name,
			'email'       => $email,
			'phone'       => $phone,
			'productinfo' => $product_info,
			'surl'        => $surl,
			'furl'        => $furl,
			'udf1'        => '',
			'udf2'        => '',
			'udf3'        => '',
			'udf4'        => '',
			'udf5'        => '',
			'address1'    => $address1,
			'address2'    => $address2,
			'city'        => $city,
			'state'       => $state,
			'country'     => $country,
			'zipcode'     => $zipcode,
		);

		$response = $easebuzzObj->initiatePaymentAPI( $postData, false );

		if ( 0 === $response->status ) {
			if ( isset( $response->error_desc ) ) {
				$this->set_error( new WP_Error( 'easebuzz_error', $response->error_desc ) );
			} else {
				$this->set_error( new WP_Error( 'easebuzz_error', $response->data ) );
			}
			return;
		}

		$accesskey = ( $response->status === 1 ) ? $response->data : null;

		$payment_link = _getURL( $this->ENV ) . 'pay/' . $accesskey;

		$payment->set_transaction_id( $txnid );
		$payment->set_action_url( $payment_link );
	}

	/**
	 * Update status of the specified payment.
	 *
	 * @param Payment $payment
	 *            Payment.
	 */
	public function update_status( Payment $payment ) {
		include_once 'easebuzz-lib/easebuzz_payment_gateway.php';

		$merchant_key  = $this->config->merchant_key;
		$merchant_salt = $this->config->merchant_salt;

		$easebuzzObj = new Easebuzz( $merchant_key, $merchant_salt, $this->ENV );

		$result = $easebuzzObj->easebuzzResponse( $_POST );
		$result = json_decode( $result, true );

		if ( 0 === $result['status'] ) {
			$this->error = new WP_Error( 'easebuzz_error', $result['data'] );
			return;
		}
		$data = $result['data'];

		if ( $data['txnid'] !== $payment->get_transaction_id() || $data['key'] !== $merchant_key ) {
			$this->error = new WP_Error( 'easebuzz_error', 'Key Missmatch!' );
			return;
		}

		$payment->set_transaction_id( $data['easepayid'] );
		$payment->set_status( Statuses::transform( $data['status'] ) );

	}
}
